@@ -1,8 +1,7 @@
 /* SPDX-License-Identifier: LGPL-2.1+ */
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE 1
-#endif
+#include "config.h"
+
 #include <arpa/inet.h>
 #include <ctype.h>
 #include <errno.h>
@@ -30,7 +29,6 @@
 #include <unistd.h>
 
 #include "compiler.h"
-#include "config.h"
 #include "file_utils.h"
 #include "log.h"
 #include "memory_utils.h"
@@ -42,8 +40,8 @@
 #include "syscall_wrappers.h"
 #include "utils.h"
 
-#ifndef HAVE_STRLCPY
-#include "include/strlcpy.h"
+#if !HAVE_STRLCPY
+#include "strlcpy.h"
 #endif
 
 #define usernic_debug_stream(stream, format, ...)                              \
@@ -111,11 +109,11 @@ static char *get_username(void)
 	__do_free char *buf = NULL;
 	struct passwd pwent;
 	struct passwd *pwentp = NULL;
-	size_t bufsize;
+	ssize_t bufsize;
 	int ret;
 
 	bufsize = sysconf(_SC_GETPW_R_SIZE_MAX);
-	if (bufsize == -1)
+	if (bufsize < 0)
 		bufsize = 1024;
 
 	buf = malloc(bufsize);
@@ -144,7 +142,7 @@ static char **get_groupnames(void)
 	int ret, i;
 	struct group grent;
 	struct group *grentp = NULL;
-	size_t bufsize;
+	ssize_t bufsize;
 
 	ngroups = getgroups(0, NULL);
 	if (ngroups < 0) {
@@ -174,7 +172,7 @@ static char **get_groupnames(void)
 	}
 
 	bufsize = sysconf(_SC_GETGR_R_SIZE_MAX);
-	if (bufsize == -1)
+	if (bufsize < 0)
 		bufsize = 1024;
 
 	buf = malloc(bufsize);
@@ -659,6 +657,7 @@ static char *get_nic_if_avail(int fd, struct alloted_s *names, int pid,
 	size_t length = 0;
 	int ret;
 	size_t slen;
+	ssize_t nbytes;
 	char *owner;
 	char nicname[IFNAMSIZ];
 	struct alloted_s *n;
@@ -755,7 +754,8 @@ static char *get_nic_if_avail(int fd, struct alloted_s *names, int pid,
 		return NULL;
 	}
 
-	if (lxc_pwrite_nointr(fd, newline, slen, length) != slen) {
+	nbytes = lxc_pwrite_nointr(fd, newline, slen, length);
+	if (nbytes < 0 || (size_t)nbytes != slen) {
 		CMD_SYSERROR("Failed to append new entry \"%s\" to database file", newline);
 
 		if (lxc_netdev_delete_by_name(nicname) != 0)
