@@ -3,11 +3,12 @@
 #ifndef __LXC_CONFILE_H
 #define __LXC_CONFILE_H
 
+#include "config.h"
+
 #include <stdbool.h>
 #include <stdio.h>
 
-#include <lxc/attach_options.h>
-#include <lxc/lxccontainer.h>
+#include "lxc.h"
 
 #include "compiler.h"
 
@@ -89,7 +90,7 @@ __hidden extern void lxc_config_define_free(struct lxc_list *defines);
  */
 __hidden extern int lxc_config_parse_arch(const char *arch, signed long *persona);
 
-__hidden extern int lxc_fill_elevated_privileges(char *flaglist, int *flags);
+__hidden extern int lxc_fill_elevated_privileges(char *flaglist, unsigned int *flags);
 
 __hidden extern int lxc_clear_config_item(struct lxc_conf *c, const char *key);
 
