@@ -1,8 +1,7 @@
 /* SPDX-License-Identifier: LGPL-2.1+ */
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE 1
-#endif
+#include "config.h"
+
 #include <inttypes.h>
 #include <linux/limits.h>
 #include <sched.h>
@@ -14,10 +13,11 @@
 #include <sys/wait.h>
 #include <unistd.h>
 
+#include "attach_options.h"
+
 #include "cgroup.h"
 #include "commands.h"
 #include "conf.h"
-#include "config.h"
 #include "criu.h"
 #include "log.h"
 #include "lxc.h"
@@ -29,13 +29,13 @@
 #include "utils.h"
 
 #if IS_BIONIC
-#include <../include/lxcmntent.h>
+#include "lxcmntent.h"
 #else
 #include <mntent.h>
 #endif
 
-#ifndef HAVE_STRLCPY
-#include "include/strlcpy.h"
+#if !HAVE_STRLCPY
+#include "strlcpy.h"
 #endif
 
 #define CRIU_VERSION		"2.0"
@@ -45,6 +45,7 @@
 
 #define CRIU_IN_FLIGHT_SUPPORT	"2.4"
 #define CRIU_EXTERNAL_NOT_VETH	"2.8"
+#define CRIU_EXTERNAL_NETDEV	"3.15"
 
 lxc_log_define(criu, lxc);
 
@@ -168,8 +169,9 @@ static int exec_criu(struct cgroup_ops *cgroup_ops, struct lxc_conf *conf,
 	char log[PATH_MAX];
 	int static_args = 23, ret;
 	int netnr = 0;
-	struct lxc_list *it;
 	struct mntent mntent;
+	struct lxc_netdev *netdev;
+	struct string_entry *strentry;
 
 	char buf[4096], ttys[32];
 
@@ -233,7 +235,7 @@ static int exec_criu(struct cgroup_ops *cgroup_ops, struct lxc_conf *conf,
 		if (ttys[0])
 			static_args += 2;
 
-		static_args += lxc_list_len(&opts->c->lxc_conf->network) * 2;
+		static_args += list_len(netdev, &opts->c->lxc_conf->netdevs, head) * 2;
 	} else {
 		return log_error_errno(-EINVAL, EINVAL, "Invalid criu operation specified");
 	}
@@ -247,7 +249,7 @@ static int exec_criu(struct cgroup_ops *cgroup_ops, struct lxc_conf *conf,
 	if (opts->user->action_script)
 		static_args += 2;
 
-	static_args += 2 * lxc_list_len(&opts->c->lxc_conf->mount_list);
+	static_args += 2 * list_len(strentry, &opts->c->lxc_conf->mount_entries, head);
 
 	ret = strnprintf(log, sizeof(log), "%s/%s.log", opts->user->directory, opts->action);
 	if (ret < 0)
@@ -321,7 +323,7 @@ static int exec_criu(struct cgroup_ops *cgroup_ops, struct lxc_conf *conf,
 
 		tmp = path_simplify(cgroup_base_path);
 		if (!tmp)
-			return log_error_errno(-ENOMEM, ENOMEM, "Failed to remove extraneous slashes from \"%s\"", tmp);
+			return log_error_errno(-ENOMEM, ENOMEM, "Failed to remove extraneous slashes from \"%s\"", cgroup_base_path);
 		free_move_ptr(cgroup_base_path, tmp);
 
 		if (controllers_list[0]) {
@@ -334,7 +336,7 @@ static int exec_criu(struct cgroup_ops *cgroup_ops, struct lxc_conf *conf,
 			WARN("No cgroup controllers configured in container's cgroup %s", cgroup_base_path);
 			ret = sprintf(buf, "%s", cgroup_base_path);
 		}
-		if (ret < 0 || ret >= sizeof(buf))
+		if (ret < 0 || (size_t)ret >= sizeof(buf))
 			return log_error_errno(-EIO, EIO, "sprintf of cgroup root arg failed");
 
 		DECLARE_ARG("--cgroup-root");
@@ -349,7 +351,7 @@ static int exec_criu(struct cgroup_ops *cgroup_ops, struct lxc_conf *conf,
 		DECLARE_ARG(opts->user->action_script);
 	}
 
-	f_mnt = make_anonymous_mount_file(&opts->c->lxc_conf->mount_list,
+	f_mnt = make_anonymous_mount_file(&opts->c->lxc_conf->mount_entries,
 	                                 opts->c->lxc_conf->lsm_aa_allow_nesting);
 	if (!f_mnt)
 		return log_error_errno(-ENOENT, ENOENT, "Failed to create anonymous mount file");
@@ -442,7 +444,7 @@ static int exec_criu(struct cgroup_ops *cgroup_ops, struct lxc_conf *conf,
 			char ghost_limit[32];
 
 			ret = sprintf(ghost_limit, "%"PRIu64, opts->user->ghost_limit);
-			if (ret < 0 || ret >= sizeof(ghost_limit))
+			if (ret < 0 || (size_t)ret >= sizeof(ghost_limit))
 				return log_error_errno(-EIO, EIO, "Failed to print ghost limit %"PRIu64, opts->user->ghost_limit);
 
 			DECLARE_ARG("--ghost-limit");
@@ -492,10 +494,9 @@ static int exec_criu(struct cgroup_ops *cgroup_ops, struct lxc_conf *conf,
 			DECLARE_ARG(buf);
 		}
 
-		lxc_list_for_each(it, &opts->c->lxc_conf->network) {
+		list_for_each_entry(netdev, &opts->c->lxc_conf->netdevs, head) {
 			size_t retlen;
 			char eth[128], *veth;
-			struct lxc_netdev *n = it->elem;
 			bool external_not_veth;
 
 			if (cmp_version(opts->criu_version, CRIU_EXTERNAL_NOT_VETH) >= 0) {
@@ -508,8 +509,8 @@ static int exec_criu(struct cgroup_ops *cgroup_ops, struct lxc_conf *conf,
 				external_not_veth = false;
 			}
 
-			if (n->name[0] != '\0') {
-				retlen = strlcpy(eth, n->name, sizeof(eth));
+			if (netdev->name[0] != '\0') {
+				retlen = strlcpy(eth, netdev->name, sizeof(eth));
 				if (retlen >= sizeof(eth))
 					return log_error_errno(-E2BIG, E2BIG, "Failed to append veth device name");
 			} else {
@@ -518,17 +519,17 @@ static int exec_criu(struct cgroup_ops *cgroup_ops, struct lxc_conf *conf,
 					return log_error_errno(-E2BIG, E2BIG, "Failed to append veth device name");
 			}
 
-			switch (n->type) {
+			switch (netdev->type) {
 			case LXC_NET_VETH:
-				veth = n->priv.veth_attr.pair;
+				veth = netdev->priv.veth_attr.pair;
 				if (veth[0] == '\0')
-					veth = n->priv.veth_attr.veth1;
+					veth = netdev->priv.veth_attr.veth1;
 
-				if (n->link[0] != '\0') {
+				if (netdev->link[0] != '\0') {
 					if (external_not_veth)
-						ret = strnprintf(buf, sizeof(buf), "veth[%s]:%s@%s", eth, veth, n->link);
+						ret = strnprintf(buf, sizeof(buf), "veth[%s]:%s@%s", eth, veth, netdev->link);
 					else
-						ret = strnprintf(buf, sizeof(buf), "%s=%s@%s", eth, veth, n->link);
+						ret = strnprintf(buf, sizeof(buf), "%s=%s@%s", eth, veth, netdev->link);
 				} else {
 					if (external_not_veth)
 						ret = strnprintf(buf, sizeof(buf), "veth[%s]:%s", eth, veth);
@@ -541,22 +542,36 @@ static int exec_criu(struct cgroup_ops *cgroup_ops, struct lxc_conf *conf,
 				TRACE("Added veth device entry %s", buf);
 				break;
 			case LXC_NET_MACVLAN:
-				if (n->link[0] == '\0')
-					return log_error_errno(-EINVAL, EINVAL, "Failed to find host interface for macvlan %s", n->name);
+				if (netdev->link[0] == '\0')
+					return log_error_errno(-EINVAL, EINVAL, "Failed to find host interface for macvlan %s", netdev->name);
 
-				ret = strnprintf(buf, sizeof(buf), "macvlan[%s]:%s", eth, n->link);
+				ret = strnprintf(buf, sizeof(buf), "macvlan[%s]:%s", eth, netdev->link);
 				if (ret < 0)
 					return log_error_errno(-EIO, EIO, "Failed to add macvlan entry");
 
 				TRACE("Added macvlan device entry %s", buf);
 
+				break;
+			case LXC_NET_PHYS:
+				if (cmp_version(opts->criu_version, CRIU_EXTERNAL_NETDEV) < 0)
+					return syserror_set(-EOPNOTSUPP, "Restoring physical network devices not supported");
+
+				if (is_empty_string(netdev->link))
+					return syserror_set(-EINVAL, "Specifying link is required");
+
+				ret = strnprintf(buf, sizeof(buf), "netdev[%s]:%s", eth, netdev->link);
+				if (ret < 0)
+					return syserror_set(-EIO, "Failed to append phys device name");
+
+				TRACE("Added phys device entry %s", buf);
 				break;
 			case LXC_NET_NONE:
+				__fallthrough;
 			case LXC_NET_EMPTY:
 				break;
 			default:
 				/* we have screened for this earlier... */
-				return log_error_errno(-EINVAL, EINVAL, "Unsupported network type %d", n->type);
+				return log_error_errno(-EINVAL, EINVAL, "Unsupported network type %d", netdev->type);
 			}
 
 			if (external_not_veth)
@@ -809,7 +824,7 @@ version_error:
  * dump. */
 static bool criu_ok(struct lxc_container *c, char **criu_version)
 {
-	struct lxc_list *it;
+	struct lxc_netdev *netdev;
 
 	if (geteuid()) {
 		ERROR("Must be root to checkpoint");
@@ -820,16 +835,16 @@ static bool criu_ok(struct lxc_container *c, char **criu_version)
 		return false;
 
 	/* We only know how to restore containers with veth networks. */
-	lxc_list_for_each(it, &c->lxc_conf->network) {
-		struct lxc_netdev *n = it->elem;
-		switch(n->type) {
+	list_for_each_entry(netdev, &c->lxc_conf->netdevs, head) {
+		switch(netdev->type) {
 		case LXC_NET_VETH:
 		case LXC_NET_NONE:
 		case LXC_NET_EMPTY:
+		case LXC_NET_PHYS:
 		case LXC_NET_MACVLAN:
 			break;
 		default:
-			ERROR("Found un-dumpable network: %s (%s)", lxc_net_type_to_str(n->type), n->name);
+			ERROR("Found un-dumpable network: %s (%s)", lxc_net_type_to_str(netdev->type), netdev->name);
 			if (criu_version) {
 				free(*criu_version);
 				*criu_version = NULL;
@@ -844,14 +859,13 @@ static bool criu_ok(struct lxc_container *c, char **criu_version)
 static bool restore_net_info(struct lxc_container *c)
 {
 	int ret;
-	struct lxc_list *it;
 	bool has_error = true;
+	struct lxc_netdev *netdev;
 
 	if (container_mem_lock(c))
 		return false;
 
-	lxc_list_for_each(it, &c->lxc_conf->network) {
-		struct lxc_netdev *netdev = it->elem;
+	list_for_each_entry(netdev, &c->lxc_conf->netdevs, head) {
 		char template[IFNAMSIZ];
 
 		if (netdev->type != LXC_NET_VETH)
