@@ -1,8 +1,7 @@
 /* SPDX-License-Identifier: LGPL-2.1+ */
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE 1
-#endif
+#include "config.h"
+
 #include <errno.h>
 #include <linux/netlink.h>
 #include <linux/rtnetlink.h>
@@ -12,7 +11,6 @@
 #include <sys/socket.h>
 #include <unistd.h>
 
-#include "config.h"
 #include "nl.h"
 #include "rtnl.h"
 
