@@ -3,19 +3,23 @@
 #ifndef __LXC_STRING_UTILS_H
 #define __LXC_STRING_UTILS_H
 
-#include <stdarg.h>
-
 #include "config.h"
 
+#include <stdarg.h>
+
 #include "initutils.h"
 #include "macro.h"
 
-#ifndef HAVE_STRLCAT
-#include "include/strlcat.h"
+#if !HAVE_STRLCAT
+#include "strlcat.h"
 #endif
 
-#ifndef HAVE_STRCHRNUL
-#include "include/strchrnul.h"
+#if !HAVE_STRLCPY
+#include "strlcpy.h"
+#endif
+
+#if !HAVE_STRCHRNUL
+#include "strchrnul.h"
 #endif
 
 /* convert variadic argument lists to arrays (for execl type argument lists) */
@@ -147,13 +151,13 @@ static inline char *deabs(char *str)
 	return str + strspn(str, "/");
 }
 
-#define strnprintf(buf, buf_size, ...)                                            \
-	({                                                                        \
-		int __ret_strnprintf;                                             \
-		__ret_strnprintf = snprintf(buf, buf_size, ##__VA_ARGS__);        \
-		if (__ret_strnprintf < 0 || (size_t)__ret_strnprintf >= buf_size) \
-			__ret_strnprintf = ret_errno(EIO);                        \
-		__ret_strnprintf;                                                 \
+#define strnprintf(buf, buf_size, ...)                                                    \
+	({                                                                                \
+		int __ret_strnprintf;                                                     \
+		__ret_strnprintf = snprintf(buf, buf_size, ##__VA_ARGS__);                \
+		if (__ret_strnprintf < 0 || (size_t)__ret_strnprintf >= (size_t)buf_size) \
+			__ret_strnprintf = ret_errno(EIO);				  \
+		__ret_strnprintf;                                                         \
 	})
 
 static inline const char *proc_self_fd(int fd)
@@ -168,12 +172,12 @@ static inline const char *proc_self_fd(int fd)
 	return buf;
 }
 
-static inline const char *fdstr(int fd)
+static inline const char *fdstr(__s64 fd)
 {
 	static const char *fdstr_invalid = "-EBADF";
-	static char buf[INTTYPE_TO_STRLEN(int)];
+	static char buf[INTTYPE_TO_STRLEN(__s64)];
 
-	if (strnprintf(buf, sizeof(buf), "%d", fd) < 0)
+	if (strnprintf(buf, sizeof(buf), "%lld", (long long signed int)fd) < 0)
 		return fdstr_invalid;
 
 	return buf;
