@@ -16,6 +16,9 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
+
+#include "config.h"
+
 #include <lxc/lxccontainer.h>
 
 #include <unistd.h>
@@ -29,6 +32,11 @@
 
 #include "lxc/state.h"
 #include "lxctest.h"
+#include "utils.h"
+
+#if !HAVE_STRLCPY
+#include "strlcpy.h"
+#endif
 
 #define MYNAME "lxctest1"
 
@@ -402,6 +410,7 @@ int main(int argc, char *argv[])
 	}
 	printf("lxc.proc returned %d %s\n", ret, v3);
 
+#if HAVE_APPARMOR
 	if (!c->set_config_item(c, "lxc.apparmor.profile", "unconfined")) {
 		fprintf(stderr, "%d: failed to set aa_profile\n", __LINE__);
 		goto out;
@@ -413,6 +422,7 @@ int main(int argc, char *argv[])
 		goto out;
 	}
 	printf("lxc.aa_profile returned %d %s\n", ret, v2);
+#endif
 
 	lxc_container_put(c);
 
